#pragma once
#ifndef _CONSTANTS_H            // Prevent multiple definitions if this 
#define _CONSTANTS_H            // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <windows.h>

//=============================================================================
// Function templates for safely dealing with pointer referenced items.
// The functions defined by these templates may be called using a normal
// function call syntax. The compiler will create a function that replaces T
// with the type of the calling parameter.
//=============================================================================
// Safely release pointer referenced item
template <typename T>
inline void safeRelease(T& ptr)
{
    if (ptr)
    { 
        ptr->Release(); 
        ptr = NULL;
    }
}
#define SAFE_RELEASE safeRelease            // for backward compatiblility

// Safely delete pointer referenced item
template <typename T>
inline void safeDelete(T& ptr)
{
    if (ptr)
    { 
        delete ptr; 
        ptr = NULL;
    }
}
#define SAFE_DELETE safeDelete              // for backward compatiblility

// Safely delete pointer referenced array
template <typename T>
inline void safeDeleteArray(T& ptr)
{
    if (ptr)
    { 
        delete[] ptr; 
        ptr = NULL;
    }
}
#define SAFE_DELETE_ARRAY safeDeleteArray   // for backward compatiblility

// Safely call onLostDevice
template <typename T>
inline void safeOnLostDevice(T& ptr)
{
    if (ptr)
        ptr->onLostDevice(); 
}
#define SAFE_ON_LOST_DEVICE safeOnLostDevice    // for backward compatiblility

// Safely call onResetDevice
template <typename T>
inline void safeOnResetDevice(T& ptr)
{
    if (ptr)
        ptr->onResetDevice(); 
}
#define SAFE_ON_RESET_DEVICE safeOnResetDevice  // for backward compatiblility

//-----------------------------------------------
//                  Constants
//-----------------------------------------------

// window
const char CLASS_NAME[] = "1.0";
const char GAME_TITLE[] = "The Squad Showdown v1.0";
const bool FULLSCREEN = false;              // windowed or fullscreen
const int  GAME_WIDTH =  1024;               // width of game in pixels
const int  GAME_HEIGHT = 768;               // height of game in pixels

// game
const bool VSYNC = false;                   // true locks display to vertical sync rate
const double PI = 3.14159265;
const float FRAME_RATE = 400.0f;                // the target frame rate (frames/sec)
const float MIN_FRAME_RATE = 10.0f;             // the minimum frame rate
const float MIN_FRAME_TIME = 1.0f/FRAME_RATE;   // minimum desired time for 1 frame
const float MAX_FRAME_TIME = 1.0f/MIN_FRAME_RATE; // maximum time used in calculations
const float GRAVITY = 6.67428e-11f;             // gravitational constant
const UINT  TILE_MAP_WIDTH = 100;           // width of tile map in tiles
const UINT  TILE_MAP_HEIGHT = 20;           // height of tile map in tiles
const RECT	BULLET_BOX = {-8,-8,8,8};
const RECT	PAUL_BOX = {-20,-20,20,20};
const RECT	SOLDIER_BOX = {-20,-20,20,20};
const RECT  CANNON_BOX = {-20,-20,20,20};


// graphic images
//const char NEBULA_IMAGE[] =   "pictures\\orion.jpg";     // photo source NASA/courtesy of nasaimages.org 
const char PAUL_IMAGE[] = "pictures\\Paul_Textures.png";
const char SOLDIER_IMAGE[] = "pictures\\Stan_Textures.png";
const char CANNON_IMAGE[] = "pictures\\Lasers.png";
const char TILES[] = "pictures\\tiles.txt";     // game tiles
const char DASHBOARD_TEXTURES[] = "pictures\\dashboard.png";
const char MENU_IMAGE[] = "pictures\\Menu.png";

// map data
const char MAP[] = "maps\\map1.txt";      // map data

// audio files required by audio.cpp
// WAVE_BANK must be location of .xwb file.
const char WAVE_BANK[]  = "audio\\Win\\TheSquad Wave Bank 2.xwb";
// SOUND_BANK must be location of .xsb file.
const char SOUND_BANK[] = "audio\\Win\\TheSquad Sound Bank 2.xsb";
// XGS_FILE must be location of .xgs file.
const char XGS_FILE[]   = "";

// audio cues

// key mappings
// In this game simple constants are used for key mappings. If variables were used
// it would be possible to save and restore key mappings from a data file.
const UCHAR CONSOLE_KEY  = '`';         // ` key
const UCHAR ESC_KEY      = VK_ESCAPE;       // escape key
const UCHAR ALT_KEY      = VK_MENU;         // Alt key
const UCHAR ENTER_KEY    = VK_RETURN;       // Enter key
const UCHAR SOLDIER_LEFT_KEY    = VK_LEFT;     // left arrow
const UCHAR SOLDIER_RIGHT_KEY   = VK_RIGHT;    // right arrow
const UCHAR SOLDIER_UP_KEY      = VK_UP;       // up arrow
const UCHAR SOLDIER_DOWN_KEY    = VK_DOWN;     // down arrow
const UCHAR SOLDIER_SHOOT_KEY	 = 'K';			// K key
const UCHAR SOLDIER_RELOAD_KEY	 = 'L';			// L key

const UCHAR PAUL_LEFT_KEY    = 'A';     // left arrow
const UCHAR PAUL_RIGHT_KEY   = 'D';    // right arrow
const UCHAR PAUL_UP_KEY      = 'W';       // up arrow
const UCHAR PAUL_DOWN_KEY    = 'S';     // down arrow
const UCHAR PAUL_SHOOT_KEY	 = 'F';			// K key
const UCHAR PAUL_RELOAD_KEY	 = 'G';			// L key

// weapon types
enum WEAPON {LASER, BULLET, SHIELD};

#endif
